@@ -303,6 +303,10 @@ our %submit_info = (
 		},
 	},
 
+	'x86_64_CentOS7'	=> 'x86_64_RedHat7',
+	'x86_64_SL7'		=> 'x86_64_RedHat7',
+
+
 	##########################################################################
 	# Platform RedHat and SL
 	##########################################################################
@@ -333,8 +337,6 @@ our %submit_info = (
 	# Add the SWAMP's (temporary) platform name
 	'swamp:rhel-6.4-64'	=> 'x86_64_RedHat6',
 
-	'x86_64_SL7'	=> 'x86_64_SL6',
-
 	# for now SL6 is the same as RedHat6
 	'x86_64_SL6'	=> 'x86_64_RedHat6',
 	'x86_64_sl_6.0' => 'x86_64_SL6',
@@ -615,6 +617,22 @@ our %submit_info = (
 		},
 	},
 
+	# Only Ubuntu 16.04 has standard universe port.
+	'x86_64_Ubuntu16'	=> {
+		'build' => {
+			'configure_args' => { @default_build_configure_args,
+				'-DCLIPPED:BOOL' => 'OFF',
+			 },
+			'prereqs'	=> [ @default_prereqs ],
+		},
+
+		'test' => {
+			'configure_args' => { @default_test_configure_args },
+			'prereqs'	=> [ @default_prereqs ],
+			'testclass'	=> [ @default_testclass ],
+		},
+	},
+
 	# Add the SWAMP's (temporary) platform name
 	'swamp:ubuntu-12.04-64'					=> 'x86_64_Ubuntu12',
 
