@@ -92,15 +92,16 @@ public:
 	DaemonList& operator = ( const DaemonList& );
 };
 
+class DCCollectorAdSequences;
 
 class CollectorList : public DaemonList {
  public:
-	CollectorList();
+	CollectorList(DCCollectorAdSequences * adseq=NULL);
 	virtual ~CollectorList();
 
 		// Create the list of collectors for the pool
 		// based on configruation settings
-	static CollectorList * create(const char * pool = NULL);
+	static CollectorList * create(const char * pool = NULL, DCCollectorAdSequences * adseq = NULL);
 
 		// Resort a collector list for locality (for negotiator)
 	int resortLocal( const char *preferred_collector );
@@ -108,6 +109,13 @@ class CollectorList : public DaemonList {
 		// Send updates to all the collectors
 		// return - number of successfull updates
 	int sendUpdates (int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblocking);
+
+		// use this to detach the ad sequence counters before destroying the collector list
+		// we do this when we want to move the sequence counters to a new list
+	DCCollectorAdSequences * detachAdSequences() { DCCollectorAdSequences * p = adSeq; adSeq = NULL; return p; }
+
+	bool hasAdSeq() { return adSeq != NULL; }
+	DCCollectorAdSequences & getAdSeq();
 	
 		// Try querying all the collectors until you get a good one
 	QueryResult query (CondorQuery & query, ClassAdList & adList, CondorError *errstack = 0);
@@ -117,6 +125,8 @@ class CollectorList : public DaemonList {
     bool next( Daemon* &);
     bool Next( Daemon* &);
 
+private:
+	DCCollectorAdSequences * adSeq;
 };
 
 
