@@ -30,54 +30,35 @@
 #include "condor_timeslice.h"
 
 #include <deque>
+#include <map>
 
-/** Class to manage the sequence nubmers of individual ClassAds
- * published by the application
- **/
+// This holds a single update ad sequence number
+//
 class DCCollectorAdSeq {
-  public:
-	DCCollectorAdSeq( const char *, const char *, const char * );
-	DCCollectorAdSeq( const DCCollectorAdSeq & );
-	~DCCollectorAdSeq( void );
-
-	bool Match( const char *, const char *, const char * ) const;
-	unsigned getSequenceAndIncrement( void );
-	unsigned getSequence( void ) const { return sequence; };
-
-	const char *getName( void ) const { return Name; };
-	const char *getMyType( void ) const { return MyType; };
-	const char *getMachine( void ) const { return Machine; };
-
-  private:
-	char		*Name;			// "Name" in the ClassAd
-	char		*MyType;		// "MyType" in the ClassAd
-	char		*Machine;		// "Machine" in ClassAd
-	unsigned	sequence;		// The sequence number for it.
+public:
+	DCCollectorAdSeq() : sequence(0), last_advance(0) {}
+	long long getSequence() { return sequence; }
+	time_t    lastAdvance() { return last_advance; }
+	long long advance(time_t now) {
+		//if ( ! now) now = time(NULL);
+		last_advance = now;
+		++sequence;
+		return sequence;
+	}
+protected:
+	long long sequence;     // current sequence number
+	time_t    last_advance; // last time advance was called (for garbage collection if we ever want to do that)
 };
 
-/** Class to manage the sequence nubmers of all ClassAds published by
- * the application
- **/
-class DCCollectorAdSeqMan {
-  public:
-	DCCollectorAdSeqMan( void );
-	DCCollectorAdSeqMan( const DCCollectorAdSeqMan &copy,
-						 bool copy_array = true );
-	~DCCollectorAdSeqMan( void );
-
-	unsigned getSequence( const ClassAd *ad );
-	int getNumAds( void ) const { return numAds; };
-
-	// Used for the copy constructor
-  protected:
-	const ExtArray<DCCollectorAdSeq *> & getSeqInfo( void ) const
-		{ return adSeqInfo; };
-
-  private:
-	ExtArray<DCCollectorAdSeq *> adSeqInfo;
-	int		numAds;
+typedef std::map<std::string, DCCollectorAdSeq> DCCollectorAdSeqMap;
+class DCCollectorAdSequences {
+public:
+	DCCollectorAdSeq* getAdSeq(const ClassAd & ad);
+private:
+	DCCollectorAdSeqMap seqs;
 };
 
+
 /** This is the Collector-specific class derived from Daemon.  It
 	implements some of the collectors's daemonCore command interface.  
 	For now, it handles sending updates to the collector, so that we
@@ -112,7 +93,7 @@ public:
 			@param ad ClassAd you want to use for the update
 			file)
 		*/
-	bool sendUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblocking );
+	bool sendUpdate( int cmd, ClassAd* ad1, DCCollectorAdSequences& seq, ClassAd* ad2, bool nonblocking );
 
 	void reconfig( void );
 
@@ -124,11 +105,6 @@ public:
 
 	bool useTCPForUpdates() { return use_tcp; }
 
-  protected:
-		// Get the ad sequence manager (for copy constructor)
-	const DCCollectorAdSeqMan &getAdSeqMan( void ) const
-		{ return *adSeqMan; };
-
 
 private:
 
@@ -166,7 +142,6 @@ private:
 
 	// Items to manage the sequence numbers
 	time_t startTime;
-	DCCollectorAdSeqMan *adSeqMan;
 };
 
 
