@@ -254,6 +254,11 @@ DCStarter::createJobOwnerSecSession(int timeout,char const *job_claim_id,char co
 {
 	ReliSock sock;
 
+	if (IsDebugLevel(D_COMMAND)) {
+		dprintf (D_COMMAND, "DCStarter::createJobOwnerSecSession(%s,...) making connection to %s\n",
+			getCommandStringSafe(CREATE_JOB_OWNER_SEC_SESSION), _addr ? _addr : "NULL");
+	}
+
 	if( !connectSock(&sock, timeout, NULL) ) {
 		error_msg = "Failed to connect to starter";
 		return false;
@@ -306,6 +311,11 @@ bool DCStarter::startSSHD(char const *known_hosts_file,char const *private_clien
 	error_msg = "This version of Condor does not support ssh key exchange.";
 	return false;
 #else
+	if (IsDebugLevel(D_COMMAND)) {
+		dprintf (D_COMMAND, "DCStarter::startSSHD(%s,...) making connection to %s\n",
+			getCommandStringSafe(START_SSHD), _addr ? _addr : "NULL");
+	}
+
 	if( !connectSock(&sock, timeout, NULL) ) {
 		error_msg = "Failed to connect to starter";
 		return false;
@@ -485,6 +495,11 @@ DCStarter::peek(bool transfer_stdout, ssize_t &stdout_offset, bool transfer_stde
 
 	ReliSock sock;
 
+	if (IsDebugLevel(D_COMMAND)) {
+		dprintf (D_COMMAND, "DCStarter::peek(%s,...) making connection to %s\n",
+			getCommandStringSafe(STARTER_PEEK), _addr ? _addr : "NULL");
+	}
+
 	if( !connectSock(&sock, timeout, NULL) ) {
 		error_msg = "Failed to connect to starter";
 		return false;
@@ -596,7 +611,7 @@ DCStarter::peek(bool transfer_stdout, ssize_t &stdout_offset, bool transfer_stde
 	if (file_count != remote_file_count)
 	{
 		std::stringstream ss;
-		ss << "Recieved " << file_count << " files, but remote side thought it sent " << remote_file_count << " files";
+		ss << "Received " << file_count << " files, but remote side thought it sent " << remote_file_count << " files";
 		error_msg = ss.str();
 		return false;
 	}
