@@ -234,24 +234,24 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 
 	args.AppendArg( dm.condorSubmitExe );
 
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString nodeName = MyString(ATTR_DAG_NODE_NAME_ALT) + " = " + DAGNodeName;
 	args.AppendArg( nodeName.Value() );
 
 		// append a line adding the parent DAGMan's cluster ID to the job ad
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString dagJobId = MyString( "+" ) + ATTR_DAGMAN_JOB_ID + " = " +
 				dm.DAGManJobId._cluster;
 	args.AppendArg( dagJobId.Value() );
 
 		// now we append a line setting the same thing as a submit-file macro
 		// (this is necessary so the user can reference it in the priority)
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString dagJobIdMacro = MyString( "" ) + ATTR_DAGMAN_JOB_ID + " = " +
 				dm.DAGManJobId._cluster;
 	args.AppendArg( dagJobIdMacro.Value() );
 
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString submitEventNotes = MyString(
 				"submit_event_notes = DAG Node: " ) + DAGNodeName;
 	args.AppendArg( submitEventNotes.Value() );
@@ -260,7 +260,7 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 
 		// We need to append the DAGman default log file to
 		// the log file list
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	std::string dlog( "dagman_log = " );
 	dlog += workflowLogFile;
 	args.AppendArg( dlog.c_str() );
@@ -269,7 +269,7 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 
 		// Now append the mask
 	debug_printf( DEBUG_VERBOSE, "Masking the events recorded in the DAGMAN workflow log\n" );
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	std::string dmask("+");
 	dmask += ATTR_DAGMAN_WORKFLOW_MASK;
 	dmask += " = \"";
@@ -283,12 +283,12 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 		// Suppress the job's log file if that option is enabled.
 	if ( dm._suppressJobLogs ) {
 		debug_printf( DEBUG_VERBOSE, "Suppressing node job log file\n" );
-		args.AppendArg( "-a" );
-		args.AppendArg( "log = ''" );
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
+		args.AppendArg( "log=" );
 	}
 
 	ArgList parentNameArgs;
-	parentNameArgs.AppendArg( "-a" );
+	parentNameArgs.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString parentNodeNames = MyString( "+DAGParentNodeNames = " ) +
 	                        "\"" + DAGParentNodeNames + "\"";
 	parentNameArgs.AppendArg( parentNodeNames.Value() );
@@ -307,20 +307,20 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 		value.replaceString( "$(RETRY)", retryStr.Value() );
 		MyString varStr = nodeVar._name + " = " + value;
 
-		args.AppendArg( "-a" );
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 		args.AppendArg( varStr.Value() );
 	}
 
 		// Set the special DAG_STATUS variable (mainly for use by
 		// "final" nodes).
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	MyString var = "DAG_STATUS = ";
 	var += dm.dag->_dagStatus;
 	args.AppendArg( var.Value() );
 
 		// Set the special FAILED_COUNT variable (mainly for use by
 		// "final" nodes).
-	args.AppendArg( "-a" );
+	args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 	var = "FAILED_COUNT = ";
 	var += dm.dag->NumNodesFailed();
 	args.AppendArg( var.Value() );
@@ -351,14 +351,14 @@ condor_submit( const Dagman &dm, const char* cmdFile, CondorID& condorID,
 	}
 
 	if( hold_claim ){
-		args.AppendArg( "-a" );
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 		MyString holdit = MyString("+") + MyString(ATTR_JOB_KEEP_CLAIM_IDLE) + " = "
 			+ dm._claim_hold_time;
 		args.AppendArg( holdit.Value() );	
 	}
 	
 	if (dm._submitDagDeepOpts.suppress_notification) {
-		args.AppendArg( "-a" );
+		args.AppendArg( "-a" ); // -a == -append; using -a to save chars
 		MyString notify = MyString("notification = never");
 		args.AppendArg( notify.Value() );
 	}
