@@ -96,7 +96,6 @@ Job::Job( const char* jobName,
 	_scriptPre = NULL;
 	_scriptPost = NULL;
 	_Status = STATUS_READY;
-	_isIdle = false;
 	countedAsDone = false;
 
 	_jobName = strnewp (jobName);
@@ -304,7 +303,46 @@ Job::SetStatus( status_t newStatus )
 	return true;
 }
 
+//---------------------------------------------------------------------------
+bool
+Job::GetProcIsIdle( int proc )
+{
+	if ( GetNoop() ) {
+		proc = 0;
+	}
 
+	if ( proc >= static_cast<int>( _isIdle.size() ) ) {
+		_isIdle.resize( proc+1, false );
+	}
+	return _isIdle[proc];
+}
+
+//---------------------------------------------------------------------------
+void
+Job::SetProcIsIdle( int proc, bool isIdle )
+{
+	if ( GetNoop() ) {
+		proc = 0;
+	}
+
+	if ( proc >= static_cast<int>( _isIdle.size() ) ) {
+		_isIdle.resize( proc+1, false );
+	}
+	_isIdle[proc] = isIdle;
+}
+
+//---------------------------------------------------------------------------
+void
+Job::PrintProcIsIdle()
+{
+	for ( int proc = 0;
+				proc < static_cast<int>( _isIdle.size() ); ++proc ) {
+		debug_printf( DEBUG_QUIET, "  Job(%s)::_isIdle[%d]: %d\n",
+					GetJobName(), proc, _isIdle[proc] );
+	}
+}
+
+//---------------------------------------------------------------------------
 bool
 Job::AddParent( Job* parent )
 {
@@ -441,7 +479,6 @@ Job::CanAddChild( Job* child, MyString &whynot )
 bool
 Job::TerminateSuccess()
 {
-	Cleanup();
 	SetStatus( STATUS_DONE );
 	return true;
 } 
@@ -449,7 +486,6 @@ Job::TerminateSuccess()
 bool
 Job::TerminateFailure()
 {
-	Cleanup();
 	SetStatus( STATUS_ERROR );
 	return true;
 } 
@@ -903,6 +939,10 @@ Job::TermAbortMetrics( int proc, const struct tm &eventTime,
 }
 
 //---------------------------------------------------------------------------
+// Note:  For multi-proc jobs, if one proc failed this was getting called
+// immediately, which was not correct.  I changed how this was called, but
+// we should probably put in code to make sure it's not called too soon,
+// but is called...  wenger 2015-11-05
 void
 Job::Cleanup()
 {
@@ -921,4 +961,7 @@ Job::Cleanup()
 
 	std::vector<unsigned char> s2;
 	_gotEvents.swap(s2); // Free memory in _gotEvents
+	
+	std::vector<unsigned char> s3;
+	_isIdle.swap(s3); // Free memory in _isIdle
 }
