@@ -180,7 +180,7 @@ HADStateMachine::isHardConfigurationNeeded(void)
 	// HAD in the list has changed or the rest of remote HAD sinful strings
 	// has changed their order or value, we do need the hard reconfiguration
 	if(  ( m_selfId     != selfId              )  ||
-		 ( !m_allHadIps.identical(m_allHadIps) )   ) {
+		 ( !m_allHadIps.identical(allHadIps) )   ) {
 		return true;
 	}
 
@@ -827,6 +827,10 @@ HADStateMachine::getHadList( const char *str,
 	Sinful my_addr( daemonCore->InfoCommandSinfulString() );
 	ASSERT( daemonCore->InfoCommandSinfulString() && my_addr.valid() );
 
+       // Don't add to the HAD list on each reconfig.
+       otherIps.clearAll();
+       allIps.clearAll();
+
     bool iAmPresent = false;
     while( (try_address = had_list.next()) ) {
         char *sinful_addr = utilToSinful( try_address );
