@@ -97,6 +97,15 @@ typedef struct macro_set {
 	MACRO_DEFAULTS * defaults; // optional reference to const defaults table
 } MACRO_SET;
 
+// this holds context during macro lookup/expansion
+typedef struct macro_eval_context {
+	const char *localname;
+	const char *subsys;
+	int without_default;
+	int use_mask;
+} MACRO_EVAL_CONTEXT;
+
+
 #if defined(__cplusplus)
 	extern MyString global_config_source;
 	extern MyString global_root_config_source;
@@ -165,23 +174,55 @@ typedef struct macro_set {
 								const char ** pdef_value,
 								const MACRO_META **ppmet);
 
-	
-	/** Look up a value by the name 'name' from the table 'table' which is table_size big.
-	
-	Values should have been inserted with insert() (above).
-	Treats name case insensitively.  Returns NULL if the name isn't in the
-	table.  On success returns a pointer to the associated value.  The
-	value is owned by the table; do not free it.
-	*/
-	MACRO_ITEM* find_macro_item (const char *name, MACRO_SET& set);
-	const char * lookup_macro (const char *name, const char *prefix, MACRO_SET& set, int use=3);
-	// lookup macro in the MACRO_SET defaults table.
-	// if the defaults table is the param table, and prefix is not NULL, then this first looks up in param subsys tables.
-	const char * lookup_macro_def(const char * name, const char* prefix, MACRO_SET& set, int use=3);
+	// lookup "name" in ALL of relevant tables as indicated by the MACRO_EVAL_CONTEXT.
+	// as soon as an item is found (even an empty item) lookup stops.  the lookup order is
+	//    localname.name in MACRO_SET
+	//    localname.name in MACRO_SET.defaults (if defaults is not null and context permits)
+	//    subsys.name in MACRO_SET
+	//    subsys.name in MACRO_SET.defaults  (if defaults is not null and context permits)
+	//    name in MACRO_SET
+	//    name in MACRO_SET.defaults  (if defaults is not null and context permits)
+	// returns:
+	//   NULL         if the macro does not exist in any of the tables
+	//   ""           if the macro exists but was not given a value.
+	//   otherwise the return value is a pointer to const memory owned by the MACRO_SET
+	//   it will be valid until the MACRO_SET is cleared (usually a reconfig)
+	//
+	const char * lookup_macro(const char * name, MACRO_SET& set, MACRO_EVAL_CONTEXT &ctx);
 
-	/*This is a faster version of lookup_macro that assumes the param name
-	  has already been prefixed with "prefix." if needed.*/
-	const char * lookup_macro_exact(const char *name, MACRO_SET& set, int use);
+	// find an item in the macro_set, but do not look in the defaults table.
+	// if prefix is not NULL, then "prefix.name" is looked up and "name" is NOT
+	// this function does not look in the defaults (param) table.
+	const char * lookup_macro_exact_no_default (const char *name, const char *prefix, MACRO_SET& set, int use=3);
+	const char * lookup_macro_exact_no_default (const char *name, MACRO_SET& set, int use=3);
+
+	// Expand parameter references of the form "left$(middle)right".  
+	// handle multiple and or nested references.
+	// Also expand references of the form "left$ENV(middle)right",
+	// replacing $ENV(middle) with getenv(middle).
+	// Also expands other $<func>() references such as $INT and $RANDOM_CHOICE
+	//
+	// value is the string to be expanded
+	// macro_set is a hash table (managed with insert() and lookup_macro)
+	// ctx   holds arguments that will be passed on to lookup_macro as needed.
+	//
+	// Returns malloc()ed memory; caller is responsible for calling free().
+	char * expand_macro(const char * value, MACRO_SET& macro_set, MACRO_EVAL_CONTEXT & ctx);
+	// expand only $(self) and $<function>(self), if ctx.subsys and/or ctx.localname is set
+	// then $(subsys.self) and/or $(localname.self) is also expanded.
+	char * expand_self_macro(const char *value, const char *self, MACRO_SET& macro_set, MACRO_EVAL_CONTEXT & ctx);
+
+	// this is the lowest level primative to doing a lookup in the macro set.
+	// it looks ONLY for an exact match of "name" in the given macro set and does
+	// not look in the defaults (param) table.
+	MACRO_ITEM* find_macro_item (const char *name, const char * prefix, MACRO_SET& set);
+
+	// lookup the macro name ONLY in the given subsys defaults table. subsys may not be null
+	const MACRO_DEF_ITEM * find_macro_subsys_def_item(const char * name, const char * subsys, MACRO_SET & set, int use);
+	// do an exact match lookup for the given name looking only in the defaults (param) table.
+	// if the name is "prefix.name" then it will look for "name" in the "prefix" defaults table
+	// OR "prefix.name" in the main defaults table.
+	const MACRO_DEF_ITEM * find_macro_def_item(const char * name, MACRO_SET & set, int use);
 
 	void optimize_macros(MACRO_SET& macro_set);
 
@@ -236,31 +277,9 @@ extern "C" {
 	char * is_valid_config_assignment(const char *config);
 	// this function allows tests to pretend that a param was set to a given value.	
     void  param_insert(const char * name, const char * value);
-	/** Expand parameter references of the form "left$(middle)right".  
-	
-	This is deceptively simple, but does handle multiple and or nested
-	references.  If self is not NULL, then we only expand references to to the
-	parameter specified by self.  This is used when we want to expand
-	self-references only.
-
-	table is a hash table (managed with insert() and macro_lookup)
-	that contains the various $(middle)s.
-
-	Also expand references of the form "left$ENV(middle)right",
-	replacing $ENV(middle) with getenv(middle).
-
-	Also expand references of the form "left$RANDOM_CHOICE(middle)right".
-
-	NOTE: Returns malloc()ed memory; caller is responsible for calling free().
-	*/
-	char * expand_macro (const char *value, MACRO_SET& macro_set,
-						 bool use_default_param_table=false,
-						 const char *subsys=NULL, int use=2);
-	// Iterator for the hash array managed by insert() and expand_macro().  See
-	// hash_iter_begin(), hash_iter_next(), hash_iter_key(), hash_iter_value(),
-	// and hash_iter_delete() for details.
 } // end extern "C"
 
+
 // the HASHITER can only be defined with c++ linkage
 class HASHITER {
 public:
@@ -293,7 +312,7 @@ the char* return value should be freed using free()
 BEGIN_C_DECLS
 char * expand_param (const char *str); // same as below but defaults subsys and use flags
 END_C_DECLS
-char * expand_param (const char *str, const char *subsys, int use);
+char * expand_param (const char *str, const char * localname, const char *subsys, int use);
 inline bool expand_param (const char *str, std::string & expanded) {
 	char * p = expand_param(str);
 	if (!p) return false;
@@ -314,6 +333,28 @@ int write_config_file(const char* pathname, int options);
 
 extern "C" {
 
+#if 1
+	/** Find next $$(MACRO) or $$([expression]) in value
+		search begins at pos and continues to terminating null
+
+	- value - The null-terminated string to scan. WILL BE MODIFIED!
+
+	- pos - 0-indexed position in value to start scanning at.
+
+	- left - OUTPUT. *leftp will be set to value+search_pos.  It
+	  will be null terminated at the first $ for the next $$(MACRO) found.
+
+	- name - OUTPUT. The name of the MACRO (the bit between the
+	  parenthesis).  Pointer into value.  Null terminated at the
+	  closing parenthesis.
+
+	- right - OUTPUT. Everything to the right of the close paren for the $$(MACRO).
+	  Pointer into value.
+
+	returns non-zero if a $$() is found, zero if not found.
+	*/
+	int next_dollardollar_macro(char * value, int pos, char** left, char** name, char** right);
+#else
 
 	/** Find next $(MACRO) or $$(MACRO) in a string
 
@@ -345,6 +386,7 @@ extern "C" {
 	int find_config_macro( register char *value, register char **leftp,
 		register char **namep, register char **rightp,
 		const char *self=NULL, bool getdollardollar=false, int search_pos=0);
+#endif
 
 	void init_config (int options);
 }
@@ -355,8 +397,7 @@ BEGIN_C_DECLS
 
 	char * get_tilde(void);
 	char * param ( const char *name );
-	char * param_without_default(const char *name);
-	char * param_with_default_abort(const char *name, int abort);
+
 	/** Insert a value into a hash table.
 
 	The value of 'value' is associated with the name 'name'.  This is
@@ -370,7 +411,7 @@ BEGIN_C_DECLS
 	extern const MACRO_SOURCE WireMacro;
 	extern const MACRO_SOURCE DetectedMacro;
 
-	void insert (const char *name, const char *value, MACRO_SET& macro_set, const MACRO_SOURCE & source);
+	void insert_macro (const char *name, const char *value, MACRO_SET& macro_set, const MACRO_SOURCE & source, MACRO_EVAL_CONTEXT & ctx);
 	inline const char * macro_source_filename(MACRO_SOURCE& source, MACRO_SET& set) { return set.sources[source.id]; }
 	
 	/** Sets the whether or not a macro has actually been used
@@ -379,7 +420,7 @@ BEGIN_C_DECLS
 	void clear_macro_use_count (const char *name, MACRO_SET& macro_set);
 	int get_macro_use_count (const char *name, MACRO_SET& macro_set);
 	int get_macro_ref_count (const char *name, MACRO_SET& macro_set);
-	bool config_test_if_expression(const char * expr, bool & result, std::string & err_reason);
+	bool config_test_if_expression(const char * expr, bool & result, const char * localname, const char * subsys, std::string & err_reason);
 
 	// populate a MACRO_SET from either a config file or a submit file.
 	#define READ_MACROS_SUBMIT_SYNTAX           0x01
@@ -404,7 +445,7 @@ BEGIN_C_DECLS
 		void * pvParseData);
 		*/
 
-	int Parse_config_string(MACRO_SOURCE& source, int depth, const char * config, MACRO_SET& macro_set, const char * subsys);
+	int Parse_config_string(MACRO_SOURCE& source, int depth, const char * config, MACRO_SET& macro_set, MACRO_EVAL_CONTEXT & ctx);
 
 	int
 	Parse_macros(
@@ -413,7 +454,7 @@ BEGIN_C_DECLS
 		int depth, // a simple recursion detector
 		MACRO_SET& macro_set,
 		int options,
-		const char * subsys,
+		MACRO_EVAL_CONTEXT * pctx,
 		std::string& config_errmsg,
 		int (*fnSubmit)(void* pv, MACRO_SOURCE& source, MACRO_SET& set, char * line, std::string & errmsg),
 		void * pvSubmitData);
