@@ -27,6 +27,7 @@
 #include "string_list.h"
 #include "dc_collector.h"
 #include "condor_ver_info.h"
+#include "matchmaker_negotiate.h"
 
 #include <vector>
 #include <string>
@@ -155,7 +156,7 @@ class Matchmaker : public Service
 
 		int update_collector_tid;
 		void updateCollector();
-
+		
 		// auxillary functions
 		bool obtainAdsFromCollector (ClassAdList&, ClassAdListDoesNotDeleteAds&, ClassAdListDoesNotDeleteAds&, ClaimIdHash& );	
 		char * compute_significant_attrs(ClassAdListDoesNotDeleteAds & startdAds);
@@ -206,6 +207,11 @@ class Matchmaker : public Service
 		void calculateNormalizationFactor (ClassAdListDoesNotDeleteAds &, double &, double &,
 										   double &, double &);
 
+		// Check to see if any concurrency limit is violated with the given set of limits.
+		// *Note* limits will be changed to lower-case.
+		bool rejectForConcurrencyLimits(std::string &limits);
+
+
 		/** Calculate a submitter's share of the pie.
 			@param quiet Do not emitt debug information about the calculation
 			@param scheddName Name attribute from the submitter ad.
@@ -282,7 +288,7 @@ class Matchmaker : public Service
 		static unsigned int HashFunc(const MyString &Key);
 		friend int comparisonFunction (AttrList *, AttrList *,
 										void *);
-		bool pslotMultiMatch(ClassAd *job, ClassAd *machine, double preemptPrio);
+		bool pslotMultiMatch(ClassAd *job, ClassAd *machine, double preemptPrio, string &dslot_claims);
 
 		/** trimStartdAds will throw out startd ads have no business being 
 			visible to the matchmaking engine, but were fetched from the 
@@ -304,7 +310,13 @@ class Matchmaker : public Service
 		/* ODBC insert functions */
 		void insert_into_rejects(char const *userName, ClassAd& job);
 		void insert_into_matches(char const *userName, ClassAd& request, ClassAd& offer);
-		
+
+			// Returns a pslot to the match list (after consumption policies have been applied).
+			// Recalculates ranks and re-sorts match list.
+			// ASSUMES NO_PREEMPTION for pslots.
+		bool returnPslotToMatchList(ClassAd &request, ClassAd *offer);
+		void calculateRanks(ClassAd &request, ClassAd *offer, PreemptState candidatePreemptState, double &candidateRankValue, double &candidatePreJobRankValue, double &candidatePostJobRankValue, double &candidatePreemptRankValue);
+
 
 		void RegisterAttemptedOfflineMatch( ClassAd *job_ad, ClassAd *startd_ad );
 
@@ -315,6 +327,7 @@ class Matchmaker : public Service
 		int  MaxTimePerCycle;		// how long for total negotiation cycle
 		int  MaxTimePerSubmitter;   // how long to talk to any one submitter
 		int  MaxTimePerSpin;        // How long per pie spin
+		int  MaxTimePerSchedd;		// How long to talk to any one schedd
 		ExprTree *PreemptionReq;	// only preempt if true
 		ExprTree *PreemptionReqPslot;	// only preempt pslots if true
 		ExprTree *PreemptionRank; 	// rank preemption candidates
@@ -338,6 +351,8 @@ class Matchmaker : public Service
 		StringList NegotiatorMatchExprNames;
 		StringList NegotiatorMatchExprValues;
 
+		map<string, int> ScheddsTimeInCycle;
+
 		CollectorList* Collectors;
 
 		typedef HashTable<MyString, MapEntry*> AdHash;
@@ -376,7 +391,8 @@ class Matchmaker : public Service
 		int rejPreemptForPolicy; //   - PREEMPTION_REQUIREMENTS == False?
 		int rejPreemptForRank;	//   - startd RANKs new job lower?
 		int rejForSubmitterLimit;   //   - not enough group quota?
-        string rejectedConcurrencyLimit; // the name of concurrency limit rejected
+	std::set<std::string> rejectedConcurrencyLimits;
+	std::string lastRejectedConcurrencyString;
 
 
 		// Class used to store each individual entry in the
@@ -397,6 +413,7 @@ class Matchmaker : public Service
 			double			PostJobRankValue;
 			double			PreemptRankValue;
 			PreemptState	PreemptStateValue;
+			MyString			DslotClaims;
 			ClassAd *ad;
 		};
 
@@ -417,7 +434,17 @@ class Matchmaker : public Service
 		{
 		public:
 
-			ClassAd* pop_candidate();
+			ClassAd* pop_candidate(string &dslot_claims);
+				// Return the previously-pop'd candidate back into the list.
+				// Note that this assumes there is empty space in the front of the list
+				// Also assume list was already sorted.
+				// Returns false if there was no space (i.e., this wasn't previously pop'd).
+			bool insert_candidate(ClassAd * candidate,
+					double candidateRankValue,
+					double candidatePreJobRankValue,
+					double candidatePostJobRankValue,
+					double candidatePreemptRankValue,
+					PreemptState candidatePreemptState);
 			bool cache_still_valid(ClassAd &request,ExprTree *preemption_req,
 				ExprTree *preemption_rank,bool preemption_req_unstable, bool preemption_rank_unstable);
 			void get_diagnostics(int & rejForNetwork,
@@ -439,7 +466,8 @@ class Matchmaker : public Service
 					double candidatePreJobRankValue,
 					double candidatePostJobRankValue,
 					double candidatePreemptRankValue,
-					PreemptState candidatePreemptState);
+					PreemptState candidatePreemptState,
+					const string &candidateDslotClaims);
 			void sort();
 			int length() { return adListLen - adListHead; }
 
@@ -450,7 +478,7 @@ class Matchmaker : public Service
 
 
 		private:
-
+			
 			// AdListEntry* peek_candidate();
 			static int sort_compare(const void*, const void*);
 			AdListEntry* AdListArray;			
