@@ -30,6 +30,7 @@
 
 #include "condor_daemon_core.h"
 #include "util_lib_proto.h"
+#include "qmgmt.h"
 #include "condor_qmgr.h"
 #include "scheduler.h"
 #include "dedicated_scheduler.h"
@@ -40,19 +41,16 @@
 #include "subsystem_info.h"
 #include "ipv6_hostname.h"
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
 #if defined(HAVE_DLOPEN)
 #include "ScheddPlugin.h"
 #include "ClassAdLogPlugin.h"
 #endif
-#endif
 
 extern "C"
 {
 	int		ReadLog(char*);
 }
-extern	void	mark_jobs_idle();
-extern  int     clear_autocluster_id( ClassAd *job );
+
 
 char*          Spool = NULL;
 char*          Name = NULL;
@@ -85,6 +83,9 @@ main_init(int argc, char* argv[])
 		switch(ptr[0][1])
 		{
 		  case 'n':
+			if (Name) {
+				free(Name);
+			}
 			Name = build_valid_daemon_name( *(++ptr) );
 			break;
 		  default:
@@ -101,27 +102,14 @@ main_init(int argc, char* argv[])
 		// each creating their own
 	daemonCore->Proc_Family_Init();
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
 #if defined(HAVE_DLOPEN)
-		// Intialization of the plugin manager, i.e. loading all
-		// plugins, should be performed before the job queue log is
-		// read so plugins have a chance to learn about all jobs
-		// already in the queue
 	ClassAdLogPluginManager::Load();
-
-		// Load all ScheddPlugins. In reality this doesn't do much
-		// since initializing any plugin manager loads plugins for all
-		// plugin manager.
 	ScheddPluginManager::Load();
 
-		// Tell all ScheddPlugins to initialze themselves
 	ScheddPluginManager::EarlyInitialize();
-
-		// Tell all plugins to initialize themselves
 	ClassAdLogPluginManager::EarlyInitialize();
 #endif
-#endif
-	
+
 		// Initialize all the modules
 	scheduler.Init();
 	scheduler.Register();
@@ -157,20 +145,7 @@ main_init(int argc, char* argv[])
 	int max_historical_logs = param_integer( "MAX_JOB_QUEUE_LOG_ROTATIONS", DEFAULT_MAX_JOB_QUEUE_LOG_ROTATIONS );
 
 	InitJobQueue(job_queue_name.Value(),max_historical_logs);
-	mark_jobs_idle();
-
-		// The below must happen _after_ InitJobQueue is called.
-	if ( scheduler.autocluster.config() ) {
-		// clear out auto cluster id attributes
-		WalkJobQueue( (int(*)(ClassAd *))clear_autocluster_id );
-	}
-	
-		//
-		// Update the SchedDInterval attributes in jobs if they
-		// have it defined. This will be for JobDeferral and
-		// CronTab jobs
-		//
-	WalkJobQueue( (int(*)(ClassAd *))::updateSchedDInterval );
+	PostInitJobQueue();
 
 		// Initialize the dedicated scheduler stuff
 	dedicated_scheduler.initialize();
@@ -178,14 +153,9 @@ main_init(int argc, char* argv[])
 		// Do a timeout now at startup to get the ball rolling...
 	scheduler.timeout();
 
-#if defined(WANT_CONTRIB) && defined(WITH_MANAGEMENT)
 #if defined(HAVE_DLOPEN)
-		// Tell all ScheddPlugins to initialze themselves
 	ScheddPluginManager::Initialize();
-
-		// Tell all plugins to initialize themselves
 	ClassAdLogPluginManager::Initialize();
-#endif
 #endif
 
 	daemonCore->InstallAuditingCallback( AuditLogNewConnection );
