@@ -72,6 +72,7 @@ BaseShadow::BaseShadow() {
 	m_cleanup_retry_tid = -1;
 	m_cleanup_retry_delay = 30;
 	m_RunAsNobody = false;
+	attemptingReconnectAtStartup = false;
 	m_force_fast_starter_shutdown = false;
 }
 
@@ -81,6 +82,7 @@ BaseShadow::~BaseShadow() {
 	if (scheddAddr) free(scheddAddr);
 	if( job_updater ) delete job_updater;
 	if (m_cleanup_retry_tid != -1) daemonCore->Cancel_Timer(m_cleanup_retry_tid);
+	free( core_file_name );
 }
 
 void
@@ -381,8 +383,22 @@ BaseShadow::reconnectFailed( const char* reason )
 	
 	logReconnectFailedEvent( reason );
 
+		// if the shadow was born disconnected, exit with 
+		// JOB_RECONNECT_FAILED so the schedd can make 
+		// an accurate restart report.  otherwise just
+		// exist with JOB_SHOULD_REQUEUE.
+	if ( attemptingReconnectAtStartup ) {
+		dprintf(D_ALWAYS,"Exiting with JOB_RECONNECT_FAILED\n");
 		// does not return
-	DC_Exit( JOB_SHOULD_REQUEUE );
+		DC_Exit( JOB_RECONNECT_FAILED );
+	} else {
+		dprintf(D_ALWAYS,"Exiting with JOB_SHOULD_REQUEUE\n");
+		// does not return
+		DC_Exit( JOB_SHOULD_REQUEUE );
+	}
+
+	// Should never get here....
+	ASSERT(true);
 }
 
 
@@ -398,7 +414,7 @@ BaseShadow::holdJob( const char* reason, int hold_reason_code, int hold_reason_s
 	}
 
 		// cleanup this shadow (kill starters, etc)
-	cleanUp();
+	cleanUp( jobWantsGracefulRemoval() );
 
 		// Put the reason in our job ad.
 	jobAd->Assign( ATTR_HOLD_REASON, reason );
@@ -481,7 +497,7 @@ void BaseShadow::removeJobPre( const char* reason )
 			 getCluster(), getProc(), reason );
 
 	// cleanup this shadow (kill starters, etc)
-	cleanUp();
+	cleanUp( jobWantsGracefulRemoval() );
 
 	// Put the reason in our job ad.
 	int size = strlen( reason ) + strlen( ATTR_REMOVE_REASON ) + 4;
@@ -722,7 +738,7 @@ BaseShadow::evictJob( int reason )
 	}
 
 		// cleanup this shadow (kill starters, etc)
-	cleanUp();
+	cleanUp( jobWantsGracefulRemoval() );
 
 		// write stuff to user log:
 	logEvictEvent( reason );
@@ -901,7 +917,6 @@ static void set_usageAd (ClassAd* jobAd, ClassAd ** ppusageAd)
 	StringList reslist(resslist.c_str());
 	if (reslist.number() > 0) {
 		ClassAd * puAd = new ClassAd();
-		puAd->Clear(); // get rid of default "CurrentTime = time()" value.
 
 		reslist.rewind();
 		while (const char * resname = reslist.next()) {
@@ -1058,7 +1073,6 @@ BaseShadow::logTerminateEvent( int exitReason, update_style_t kind )
 	if (reslist.number() > 0) {
 		int64_t int64_value = 0;
 		ClassAd * puAd = new ClassAd();
-		puAd->Clear(); // get rid of default "CurrentTime = time()" value.
 
 		reslist.rewind();
 		char * resname = NULL;
@@ -1109,7 +1123,7 @@ BaseShadow::logEvictEvent( int exitReason )
 		break;
 	default:
 		dprintf( D_ALWAYS, 
-				 "logEvictEvent with unknown reason (%d), aborting\n",
+				 "logEvictEvent with unknown reason (%d), not logging.\n",
 				 exitReason ); 
 		return;
 	}
