@@ -2248,9 +2248,11 @@ JICShadow::job_lease_expired()
 		ASSERT( !syscall_sock->is_connected() );
 	}
 
-	// Exit telling the startd we lost the shadow, which
-	// will normally result in the This does not return.
-	Starter->StarterExit(STARTER_EXIT_LOST_SHADOW_CONNECTION);
+	// Exit telling the startd we lost the shadow
+	Starter->SetShutdownExitCode(STARTER_EXIT_LOST_SHADOW_CONNECTION);
+	if ( Starter->RemoteShutdownFast(0) ) {
+		Starter->StarterExit( Starter->GetShutdownExitCode() );
+	}
 }
 
 bool
