@@ -36,6 +36,7 @@
 #include "jic_local_file.h"
 #include "jic_local_schedd.h"
 #include "vm_proc.h"
+#include "docker_proc.h"
 #include "condor_getcwd.h"
 
 
@@ -137,12 +138,36 @@ printClassAd( void )
 	if( VMProc::vm_univ_detect() ) {
 		// This doesn't mean that vm universe is really available.
 		// This just means that starter has codes for vm universe.
-		// Actual testing for vm universe will be 
+		// Actual testing for vm universe will be
 		//  done by vmuniverse manager in startd.
 		// ATTR_HAS_VM may be overwritten by vmuniverse manager in startd
-		printf( "%s = True\n",ATTR_HAS_VM);		
+		printf( "%s = True\n",ATTR_HAS_VM );
 	}
 
+	// Docker "universe."
+	if( DockerProc::Detect() ) {
+		printf( "%s = True\n", ATTR_HAS_DOCKER );
+
+		std::string dockerVersion;
+		if( DockerProc::Version( dockerVersion ) ) {
+			printf( "%s = \"%s\"\n", ATTR_DOCKER_VERSION, dockerVersion.c_str() );
+			// CondorVersion seems like a good idea to ignore;
+			// ignoring IsDaemonCore to get the same ad as before,
+			// except with ATTR_DOCKER_VERSION in it.  *sigh*
+			printf( "%s = \"CondorVersion, IsDaemonCore\"\n", ATTR_STARTER_IGNORED_ATTRS );
+		}
+	}
+
+	// Detect ability to encrypt execute directory
+#ifdef LINUX
+	if ( FilesystemRemap::EncryptedMappingDetect() ) {
+		printf( "%s = True\n", ATTR_HAS_ENCRYPT_EXECUTE_DIRECTORY );
+	}
+#endif
+#ifdef WIN32
+	printf( "%s = True\n", ATTR_HAS_ENCRYPT_EXECUTE_DIRECTORY );
+#endif
+
 	// Advertise which file transfer plugins are supported
 	FileTransfer ft;
 	CondorError e;
@@ -202,7 +227,7 @@ main_pre_dc_init( int argc, char* argv[] )
 		// up calling functions that rely on it being defined...  
 	if( argc == 2 && strncasecmp(argv[1],"-cla",4) == MATCH ) {
 			// needed for Java stuff
-		config(true);
+		config_ex(CONFIG_OPT_WANT_QUIET);
 
 			// Would like to log errors to stderr if stderr is not
 			// /dev/null to make it easier for users to debug, but not
@@ -355,6 +380,7 @@ parseArgs( int argc, char* argv [] )
 	char _header[] = "-header";
 	char _gridshell[] = "-gridshell";
 	char _schedd_addr[] = "-schedd-addr";
+	char _slot_name[] = "-slot-name";
 	char* target = NULL;
 
 	ASSERT( argc >= 2 );
@@ -377,6 +403,9 @@ parseArgs( int argc, char* argv [] )
 			if( ! arg ) {
 				another( _header );
 			}
+			if (dprintf_header) {
+				free(dprintf_header);
+			}
 			dprintf_header = strdup( arg );
 			DebugId = display_dprintf_header;
 			tmp++;	// consume the arg so we don't get confused 
@@ -400,6 +429,16 @@ parseArgs( int argc, char* argv [] )
 			continue;
 		}
 
+		if (opt_len >=4 && ! strncmp(opt, _slot_name, opt_len)) {
+			if( ! arg ) {
+				another( _slot_name );
+			}
+			config_insert( "STARTER_SLOT_NAME", arg );
+			dprintf( D_ALWAYS, "STARTER_SLOT_NAME is %s\n", arg);
+			tmp++; // consume the arg
+			continue;
+		}
+
 		if( strncmp( "-job-", opt, MIN(opt_len,5)) ) {
 			invalid( opt );
 		}
@@ -709,7 +748,7 @@ main_shutdown_fast()
 	if ( Starter->RemoteShutdownFast(0) ) {
 		// ShutdownFast says it is already finished, because there are
 		// no jobs to shutdown.  No need to stick around.
-		Starter->StarterExit(0);
+		Starter->StarterExit(Starter->GetShutdownExitCode());
 	}
 }
 
@@ -720,7 +759,7 @@ main_shutdown_graceful()
 	if ( Starter->RemoteShutdownGraceful(0) ) {
 		// ShutdownGraceful says it is already finished, because
 		// there are no jobs to shutdown.  No need to stick around.
-		Starter->StarterExit(0);
+		Starter->StarterExit(Starter->GetShutdownExitCode());
 	}
 }
 
