@@ -643,7 +643,7 @@ VanillaProc::StartJob()
 				int VMemKb;
 				if (MachineAd->LookupInteger(ATTR_VIRTUAL_MEMORY, VMemKb)) {
 
-					uint64_t memsw_limit = 1024 * VMemKb;
+					uint64_t memsw_limit = ((uint64_t)1024) * VMemKb;
 					if (VMemKb > 0) {
 						// we're not allowed to set memsw limit <
 						// the hard memory limit.  If we haven't set the hard
@@ -1008,6 +1008,7 @@ VanillaProc::outOfMemoryEvent(int /* fd */)
 	 */
 
 	// If we have no jobs left, prolly just cgroup removed, so do nothing and return
+	
 	if (num_pids == 0) {
 		dprintf(D_FULLDEBUG, "Closing event FD pipe %d.\n", m_oom_efd);
 		cleanupOOM();
@@ -1022,6 +1023,33 @@ VanillaProc::outOfMemoryEvent(int /* fd */)
 	PublishUpdateAd( &updateAd );
 	Starter->jic->periodicJobUpdate( &updateAd, true );
 
+		//
+#ifdef LINUX
+	if (param_boolean("IGNORE_LEAF_OOM", false)) {
+		// if memory.use_hierarchy is 1, then hitting the limit at
+		// the parent notifies all children, even if those children
+		// are below their usage.  If we are below our usage, ignore
+		// the OOM, and continue running.  Hopefully, some process
+		// will be killed, and when it does, this job will get unfrozen
+		// and continue running.
+
+		int usage;
+		updateAd.LookupInteger(ATTR_MEMORY_USAGE, usage);
+
+		if (usage < (0.9 * m_memory_limit)) {
+			long long oomData = 0xdeadbeef;
+			int efd;
+			daemonCore->Get_Pipe_FD(m_oom_efd, &efd);
+				// need to drain notification fd, or it will still
+				// be hot, and we'll come right back here again
+			int r = read(efd, &oomData, 8);
+
+			dprintf(D_ALWAYS, "Spurious OOM event, usage is %d, slot size is %d megabytes, ignoring OOM (read %d bytes)\n", usage, m_memory_limit, r);
+			return 0;
+		}
+	}
+#endif
+
 	std::stringstream ss;
 	if (m_memory_limit >= 0) {
 		ss << "Job has gone over memory limit of " << m_memory_limit << " megabytes.";
@@ -1246,7 +1274,7 @@ VanillaProc::setupOOMEvent(const std::string &cgroup_string)
 	m_oom_efd = pipes[0];
 	m_oom_efd2 = pipes[1];
 
-	// Inform DC we want to recieve notifications from this FD.
+	// Inform DC we want to receive notifications from this FD.
 	if (-1 == daemonCore->Register_Pipe(pipes[0],"OOM event fd", static_cast<PipeHandlercpp>(&VanillaProc::outOfMemoryEvent),"OOM Event Handler",this,HANDLE_READ))
 	{
 		dprintf(D_ALWAYS, "Failed to register OOM event FD pipe.\n");
