@@ -712,10 +712,6 @@ MultiLogFiles::logFileNFSError(const char *logFilename, bool nfsIsError)
 		if ( nfsIsError ) {
 			dprintf(D_ALWAYS, "ERROR: log file %s is on NFS.\n", logFilename);
 			return true;
-		} else {
-			dprintf(D_FULLDEBUG, "WARNING: log file %s is on NFS.  This "
-				"could cause log file corruption and is _not_ recommended.\n",
-				logFilename);
 		}
 	}
 
