@@ -341,7 +341,7 @@ bool GetAccessToken( const string &auth_file, string &access_token,
 		auth_entry.m_auth_file = auth_file;
 	}
 
-	if ( auth_entry.m_expiration < time(NULL) - (5 * 60) ) {
+	if ( auth_entry.m_expiration < time(NULL) + (5 * 60) ) {
 		auth_entry.m_refreshing = true;
 		auth_entry.m_err_msg.clear();
 
@@ -843,6 +843,7 @@ bool GceInstanceInsert::workerFunction(char **argv, int argc, string &result_str
 	insert_request.requestBody += "\",\n";
 	insert_request.requestBody += " \"disks\": [\n  {\n";
 	insert_request.requestBody += "   \"boot\": true,\n";
+	insert_request.requestBody += "   \"autoDelete\": true,\n";
 	insert_request.requestBody += "   \"initializeParams\": {\n";
 	insert_request.requestBody += "    \"sourceImage\": \"";
 	insert_request.requestBody += argv[8];
