@@ -1,4 +1,6 @@
 
+#include "python_bindings_common.h"
+
 #include <boost/python.hpp>
 
 #include "old_boost.h"
@@ -6,7 +8,6 @@
 
 using namespace boost::python;
 
-
 BOOST_PYTHON_MODULE(htcondor)
 {
     scope().attr("__doc__") = "Utilities for interacting with the HTCondor system.";
@@ -23,5 +24,11 @@ BOOST_PYTHON_MODULE(htcondor)
     export_schedd();
     export_dc_tool();
     export_secman();
+#if !defined(WIN32)
+	// omit for windows
     export_event_reader();
+    export_log_reader();
+#endif
+
+    def("enable_classad_extensions", enable_classad_extensions, "Register the HTCondor-specific extensions to the ClassAd library.");
 }
